<?php
namespace Shopware\Components\Api\Resource;

use \Doctrine\ORM\Query\Expr\Join;

/**
 * @category  Shopware
 * @package   Shopware\Plugins\rapi1RapidExtendApi
 */
class Recipient extends Resource {

    /**
     * @var string
     */
    const MAPPING_EMAIL = 'email';

    /**
     * @var string
     */
    const MAPPING_IS_CUSTOMER = 'isCustomer';

    /**
     * @var string
     */
    const MAPPING_NUMBER = 'number';

    /**
     * @var string
     */
    const MAPPING_ACTIVE = 'active';

    /**
     * @var string
     */
    const MAPPING_SHOP_ID = 'shopId';

    /**
     * @var string
     */
    const MAPPING_GROUP_KEY = 'groupKey';

    /**
     * @var string
     */
    const MAPPING_ACCOUNT_MODE = 'accountMode';

    /**
     * @var string
     */
    const MAPPING_PAYMENT_ID = 'paymentId';

    /**
     * @var string
     */
    const MAPPING_PAYMENT_PRESET = 'paymentPreset';

    /**
     * @var string
     */
    const MAPPING_PRICE_GROUP_ID = 'priceGroupId';

    /**
     * @var string
     */
    const MAPPING_BIRTHDAY = 'birthday';

    /**
     * @var string
     */
    const MAPPING_FIRST_LOGIN = 'firstLogin';

    /**
     * @var string
     */
    const MAPPING_LAST_LOGIN = 'lastLogin';

    /**
     * @var string
     */
    const MAPPING_VALIDATION = 'validation';

    /**
     * @var string
     */
    const MAPPING_LANGUAGE_ID = 'languageId';

    /**
     * @var string
     */
    const MAPPING_NEWSLETTER = 'newsletter';

    /**
     * @var string
     */
    const MAPPING_CUSTOMER_ID = 'customerId';

    /**
     * @var string
     */
    const MAPPING_ID = 'id';

    /**
     * @var string
     */
    const MAPPING_SALUTATION = 'salutation';


    /**
     * @var string
     */
    const MAPPING_FIRST_NAME = 'firstName';

    /**
     * @var string
     */
    const MAPPING_LAST_NAME = 'lastName';

    /**
     * @var string
     */
    const MAPPING_STREET = 'street';

    /**
     * @var string
     */
    const MAPPING_ZIP_CODE = 'zipCode';

    /**
     * @var string
     */
    const MAPPING_CITY = 'city';

    /**
     * @var string
     */
    const MAPPING_COUNTRY_ID = 'countryId';

    /**
     * @var string
     */
    const MAPPING_PHONE = 'phone';

    /**
     * @var string
     */
    const MAPPING_COMPANY = 'company';

    /**
     * @var string
     */
    const MAPPING_BILLING = 'billing';

    /**
     * @var string
     */
    const MAPPING_BILLING_SALUTATION = 'billingSalutation';

    /**
     * @var string
     */
    const MAPPING_BILLING_FIRST_NAME = 'billingFirstName';

    /**
     * @var string
     */
    const MAPPING_BILLING_LAST_NAME = 'billingLastName';

    /**
     * @var string
     */
    const MAPPING_BILLING_STREET = 'billingStreet';

    /**
     * @var string
     */
    const MAPPING_BILLING_ZIP_CODE = 'billingZipCode';

    /**
     * @var string
     */
    const MAPPING_BILLING_CITY = 'billingCity';

    /**
     * Calculate count for subscribed recipients and customers
     *
     * @return array
     */
    public function getRecipientsCount() {

        $count = $this
            ->getSelectionBuilder()
            ->addSelect([
                'COUNT(1)'
            ])
            ->getQuery()
            ->getSingleScalarResult();

        return ['data' => [], 'total' => $count];

    }

    /**
     * Retrieve subscribed recipients and customers
     *
     * @param int $offset
     * @param int $limit
     * @return array
     */
    public function getRecipients($offset = 0, $limit = 100) {

        $recipients = $this
            ->getSelectionBuilder()
            ->addSelect(
                $this->buildSelect()
            )
            ->setFirstResult($offset)
            ->setMaxResults($limit)
            ->getQuery()
            ->getArrayResult();

        return ['data' => $this->postProcessData($recipients), 'total' => count($recipients)];

    }

    /**
     *
     * @return string[]
     */
    private function buildSelect() {

        $selectExpression = [
            // Retrieved by model "Shopware\Models\Newsletter\Address":
            'address.email AS ' . self::MAPPING_EMAIL,
            'address.isCustomer AS ' . self::MAPPING_IS_CUSTOMER, // internal
            // Retrieved by model "Shopware\Models\Newsletter\ContactData":
            'contactData.salutation AS ' . self::MAPPING_SALUTATION,
            'contactData.firstName AS ' . self::MAPPING_FIRST_NAME,
            'contactData.lastName AS ' . self::MAPPING_LAST_NAME,
            'contactData.street AS ' . self::MAPPING_STREET,
            'contactData.zipCode AS ' . self::MAPPING_ZIP_CODE,
            'contactData.city AS ' . self::MAPPING_CITY,
            // Retrieved by model "Shopware\Models\Customer\Customer":
            'customer.id AS ' . self::MAPPING_CUSTOMER_ID,
            'customer.active AS ' . self::MAPPING_ACTIVE,
            'customer.shopId AS ' . self::MAPPING_SHOP_ID,
            'customer.groupKey AS ' . self::MAPPING_GROUP_KEY,
            'customer.accountMode AS ' . self::MAPPING_ACCOUNT_MODE,
            'customer.paymentId AS ' . self::MAPPING_PAYMENT_ID,
            'customer.paymentPreset AS ' . self::MAPPING_PAYMENT_PRESET,
            'customer.priceGroupId AS ' . self::MAPPING_PRICE_GROUP_ID,
            'customer.firstLogin AS ' . self::MAPPING_FIRST_LOGIN,
            'customer.lastLogin AS ' . self::MAPPING_LAST_LOGIN,
            'customer.validation AS ' . self::MAPPING_VALIDATION,
            'customer.languageId AS ' . self::MAPPING_LANGUAGE_ID,
            // > = 5.4 Shopware\Models\Customer\Address will be used, otherwise Shopware\Models\Customer\Billing
            'billingAddress.salutation AS ' . self::MAPPING_BILLING_SALUTATION,
            'billingAddress.street AS ' . self::MAPPING_BILLING_STREET,
            'billingAddress.city AS ' . self::MAPPING_BILLING_CITY,
            'billingAddress.phone AS ' . self::MAPPING_PHONE,
            'billingAddress.company AS ' . self::MAPPING_COMPANY,
            'billingAddress.countryId AS ' . self::MAPPING_COUNTRY_ID, // added to customer address in v5.2.7

        ];

        if (\Shopware::VERSION === '___VERSION___' || version_compare(\Shopware::VERSION, '5.2.0', '>=')) {

            $selectExpression = array_merge(
                $selectExpression,
                [
                    'customer.number AS ' . self::MAPPING_NUMBER, // moved to customer
                    'customer.birthday AS ' . self::MAPPING_BIRTHDAY, // moved to customer
                ]
            );

        } else {

            $selectExpression = array_merge(
                $selectExpression,
                [
                    'billingAddress.number AS ' . self::MAPPING_NUMBER,
                    'billingAddress.birthday AS ' . self::MAPPING_BIRTHDAY
                ]
            );

        }

        // Some properties are non camel cased when using customer address model over billing model

        if (\Shopware::VERSION === '___VERSION___' || version_compare(\Shopware::VERSION, '5.4.0', '>=')) {

            $selectExpression = array_merge(
                $selectExpression,
                [
                    'billingAddress.firstname AS ' . self::MAPPING_BILLING_FIRST_NAME,
                    'billingAddress.lastname AS ' . self::MAPPING_BILLING_LAST_NAME,
                    'billingAddress.zipcode AS ' . self::MAPPING_BILLING_ZIP_CODE,
                ]
            );

        } else {

            $selectExpression = array_merge(
                $selectExpression,
                [
                    'billingAddress.firstName AS ' . self::MAPPING_BILLING_FIRST_NAME,
                    'billingAddress.lastName AS ' . self::MAPPING_BILLING_LAST_NAME,
                    'billingAddress.zipCode AS ' . self::MAPPING_BILLING_ZIP_CODE,
                ]
            );

        }

        return $selectExpression;

    }

    /**
     * Get query builder with base tables
     *
     * @return \Doctrine\ORM\QueryBuilder
     */
    private function getSelectionBuilder() {

        $builder = $this
            ->getManager()
            ->createQueryBuilder()
            ->select()
            // Full list with all the newsletter subscribers
            ->from('Shopware\Models\Newsletter\Address', 'address')
            // Plugins may use these additional subscriber data fields for subscriber personalization (optional)
            ->leftJoin('Shopware\Models\Newsletter\ContactData', 'contactData', Join::WITH, 'contactData.email = address.email')
            // Customers, that subscribed to the newsletter (via frontend checkbox within customer data edit page)
            ->leftJoin('address.customer', 'customer', Join::WITH, 'address.isCustomer = true');

        if (\Shopware::VERSION === '___VERSION___' || version_compare(\Shopware::VERSION, '5.4.0', '>=')) {
            // Customers address data (formerly "customer.billing" was used, but is marked deprecated)
            $builder->leftJoin('customer.defaultBillingAddress', 'billingAddress');
        } else {
            // Billing data
            $builder->leftJoin('Shopware\Models\Customer\Billing', 'billingAddress', Join::WITH,'billingAddress.customerId = customer.id');
        }

        return $builder;

    }

    /**
     * Format query output to fit rapidmail datalink processor
     *
     * @param array $recipients
     * @return array
     */
    private function postProcessData($recipients = []) {

        // Map customer billing data fields to response data fields (newsletter recipient is customer)

        $customerFieldMap = [
            self::MAPPING_EMAIL => self::MAPPING_EMAIL,
            self::MAPPING_ACTIVE => self::MAPPING_ACTIVE,
            self::MAPPING_SHOP_ID => self::MAPPING_SHOP_ID,
            self::MAPPING_GROUP_KEY => self::MAPPING_GROUP_KEY,
            self::MAPPING_ACCOUNT_MODE => self::MAPPING_ACCOUNT_MODE,
            self::MAPPING_PAYMENT_ID => self::MAPPING_PAYMENT_ID,
            self::MAPPING_PAYMENT_PRESET => self::MAPPING_PAYMENT_PRESET,
            self::MAPPING_PRICE_GROUP_ID => self::MAPPING_PRICE_GROUP_ID,
            self::MAPPING_FIRST_LOGIN => self::MAPPING_FIRST_LOGIN,
            self::MAPPING_LAST_LOGIN => self::MAPPING_LAST_LOGIN,
            self::MAPPING_VALIDATION => self::MAPPING_VALIDATION,
            self::MAPPING_LANGUAGE_ID => self::MAPPING_LANGUAGE_ID,
            self::MAPPING_CUSTOMER_ID => self::MAPPING_CUSTOMER_ID,
            self::MAPPING_SALUTATION => self::MAPPING_BILLING_SALUTATION,
            self::MAPPING_FIRST_NAME => self::MAPPING_BILLING_FIRST_NAME,
            self::MAPPING_LAST_NAME => self::MAPPING_BILLING_LAST_NAME,
            self::MAPPING_STREET => self::MAPPING_BILLING_STREET,
            self::MAPPING_ZIP_CODE => self::MAPPING_BILLING_ZIP_CODE,
            self::MAPPING_CITY => self::MAPPING_BILLING_CITY,
            self::MAPPING_COUNTRY_ID => self::MAPPING_COUNTRY_ID,
            self::MAPPING_PHONE => self::MAPPING_PHONE,
            self::MAPPING_COMPANY => self::MAPPING_COMPANY,
            self::MAPPING_NUMBER => self::MAPPING_NUMBER,
            self::MAPPING_BIRTHDAY => self::MAPPING_BIRTHDAY
        ];

        // Map newsletter address data fields to response data fields (newsletter recipient is not a customer)

        $recipientFieldMap = array_combine(
            array_keys($customerFieldMap),
            array_keys($customerFieldMap)
        );

        $recipientsProcessed = [];

        foreach($recipients AS $recipient) {

            $customerInfos = [
                self::MAPPING_NEWSLETTER => true
            ];

            $map = $recipient[self::MAPPING_IS_CUSTOMER] ? $customerFieldMap : $recipientFieldMap;

            foreach ($map as $key => $value) {
                $customerInfos[$key] = $recipient[$value];
            }

            $customerInfos[self::MAPPING_EMAIL] = mb_strtolower($recipient[self::MAPPING_EMAIL]);
            $recipientsProcessed[] = $customerInfos;

        }

        return $recipientsProcessed;

    }

}
